//this file is part of notepad++
//Copyright (C)2003 Don HO ( donho@altern.org )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// This compiles under Dev-C++ 4.9.9.2 as a .c or .cpp file

// Exclude rarely-used stuff from Windows headers
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "PluginInterface.h"

#ifndef NELEM
#define NELEM(xyzzy) (sizeof(xyzzy)/sizeof(xyzzy[0]))
#endif
// This is the name which will be displayed in Plugins Menu
#ifdef __cplusplus
#define PLUGIN_NAME "Plugin Demo (C++)"
#else
#define PLUGIN_NAME "Plugin Demo (C)"
#endif
struct NppData nppData;

//----------------------------------------------------------//
// Here're the definition of functions for our functionalities
//----------------------------------------------------------//

PFUNCPLUGINCMD pftoto() {
	MessageBox(nppData._nppHandle, "tOTo", "", MB_OK);
}

PFUNCPLUGINCMD pftiti() {
	MessageBox(NULL, "tITi", "", MB_OK);
}

PFUNCPLUGINCMD pftata() {
	MessageBox(nppData._nppHandle, "tATa", "", MB_OK);
}

PFUNCPLUGINCMD pfabout() {
	MessageBox(nppData._nppHandle,
   "Are there things you want in an editor but can't find? This\r\n"
   "Notepad++ Plugin is a DLL written in C or C++. It doesn't do\r\n"
   "much but you can add to it to give the editor the functionality\r\n"
   "you want. The editor and plugin source code are available at\r\n"
   "http://notepad-plus.sourceforge.net/\r\n"
   "with which you can make your own plugins.\r\n"
   "If you're a programmer, you'll discover that your favorite editor\r\n"
   "is not the one with the most features, it's the one you can write a\r\n"
   "plugin to add your own features."
   , PLUGIN_NAME, MB_OK);
}
struct FuncItem funcItem[]={
{"toto",pftoto,0,FALSE},
{"titi",pftiti,0,FALSE},
{"tata",pftata,0,FALSE},
{"About",pfabout,0,FALSE},
};

LRESULT DLLIMPORT __cdecl messageProc(UINT Message, WPARAM wParam, LPARAM lParam) {
  return(0);
}

#if 0
/* Enabling this will help you to figure out where your compiler is going wrong
   Between the C and CPP versions, sometimes one works and the other doesn't. */
void sizetest(char *loc) {
    char msg[64];
    wsprintf(msg,"%s:%d",loc,sizeof(funcItem[0]));
	MessageBox(NULL, msg, "Location:Size of Struct", MB_OK);
}
#else
#define sizetest(u)
#endif

/*
 *--------------------------------------------------
 * The 4 extern functions are mandatory
 * They will be called by Notepad++ plugins system
 *--------------------------------------------------
*/

// The setInfo function gets the needed infos from Notepad++ plugins system
void DLLIMPORT __cdecl setInfo(struct NppData notpadPlusData) {
  sizetest("setInfo");
	nppData = notpadPlusData;
}

// The getName function tells Notepad++ plugins system its name
const char DLLIMPORT * __cdecl getName(void) {
  sizetest("getName");
	return PLUGIN_NAME;
}

// The getFuncsArray function gives Notepad++ plugins system the pointer FuncItem Array
// and the size of this array (the number of functions)
struct FuncItem DLLIMPORT * __cdecl getFuncsArray(int *nbF) {
  sizetest("getFuncsArray");
	*nbF = NELEM(funcItem);
	return funcItem;
}

// If you don't need get the notification from Notepad++,
// just let it be empty.
void DLLIMPORT __cdecl beNotified(struct SCNotification *notifyCode) {
}

//----------------------------------------------------------//
// Standard Windows DLL Entry Point
//----------------------------------------------------------//

BOOL APIENTRY DllMain (HINSTANCE hInst     /* Library instance handle. */ ,
                       DWORD reason        /* Reason this function is being called. */ ,
                       LPVOID reserved     /* Not used. */ ) {
  sizetest("DllMain");
  switch (reason) {
  case DLL_PROCESS_ATTACH:
    break;
  case DLL_PROCESS_DETACH:
    break;
  case DLL_THREAD_ATTACH:
    break;
  case DLL_THREAD_DETACH:
    break;
  }
  return TRUE;
}
/* Execution sequence:
  1) DllMain is called when Notepad++ loads the plugin
  2) N++ calls getFuncsArray, setInfo, getName to get initial operating info
  3) beNotified is called constantly while editing.
  4) Occasionally the user will select a plugin menu item.
  5) The user will quit N++ which will call DllMain for the last time. */
